#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: ngamsGenCrc32.py,v 1.20 2004/08/04 16:45:30 ngasmgr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  23/01/2002  Created
#

"""
Checksum Plug-In to generate the checksum stored in the ngas_files tables
in connection with each file archived into NGAS.
"""

import sys, time

import binascii
from   ngams import *


def ngamsGenCrc32(srvObj,
                  filename,
                  priority = 0):
    """
    Plug-in to generate CRC-32 checksum for an archived data file.

    srvObj:       Reference to instance of NG/AMS Server class (ngamsServer).

    filename:     Name of file to generate checksum for (string).

    priority:     Is used by NG/AMS to make the plug-in consume less
                  CPU. A value of 0 means highest priority (integer/[0; oo]).

    Returns:      CRC-32 checksum for file (string).
    """
    fo = open(filename, "r")
    buf = fo.read(524288)
    crc = 0
    while (buf != ""):
        crc = binascii.crc32(buf, crc)
        if (priority): time.sleep(priority * 0.001)
        buf = fo.read(524288)
    fo.close()
    return str(crc)


if __name__ == '__main__':
    """
    Main routine to calculate checksum for a file given as input parameter.
    """
    if (len(sys.argv) != 2):
        print "\nCorrect usage is:\n"
        print "% ngamsGenCrc32.py <filename> [<priority [0..oo]>]\n"
        sys.exit(1)
    filename = sys.argv[1]
    if (len(sys.argv) == 3):
        priority = int(sys.argv[2])
    else:
        priority = 0
    checksum = ngamsGenCrc32(None, filename, priority)
    sys.stdout.write(checksum)


# EOF
