#******************************************************************************
# ESO/DMD
#
# "@(#) $Id: ngamsLinuxOfflinePlugIn.py,v 1.24 2005/08/30 12:51:17 jknudstr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  10/05/2001  Created.
#

"""
Module that contains a System Offline Plug-In used by the ESO NGAS
installations.
"""

from   ngams import *
import ngamsPlugInApi
import ngamsLinuxSystemPlugInApi, ngamsEscaladeUtils


def ngamsLinuxOfflinePlugIn(srvObj,
                            reqPropsObj = None):
    """
    Function unmounts all NGAMS disks and removes the kernel module for
    the IDE controller card.

    srvObj:        Reference to instance of the NG/AMS Server class
                   (ngamsServer).

    reqPropsObj:   NG/AMS request properties object (ngamsReqProps).

    Returns:       Void.
    """
    rootMtPt = srvObj.getCfg().getMountRootDirectory()    
    parDicOnline = ngamsPlugInApi.\
                   parseRawPlugInPars(srvObj.getCfg().getOnlinePlugInPars())

    # Old format = unfortunately some Disk IDs of WDC/Maxtor were
    # generated wrongly due to a mistake by IBM, which lead to a wrong
    # implementation of the generation of the Disk ID.
    if (not parDicOnline.has_key("old_format")):
        raise Exception, "Missing Online Plug-In Parameter: old_format=0|1"
    else:
        oldFormat = int(parDicOnline["old_format"])

    # The controllers Plug-In Parameter, specifies the number of controller
    # in the system.
    if (not parDicOnline.has_key("controllers")):
        controllers = None
    else:
        controllers = parDicOnline["controllers"]

    # Select between 3ware WEB Interface and 3ware Command Line Tool.
    if (parDicOnline["uri"].find("http") != -1):
        diskDic = ngamsEscaladeUtils.\
                  parseHtmlInfo(parDicOnline["uri"], rootMtPt)
    else:
        diskDic = ngamsEscaladeUtils.parseCmdLineInfo(rootMtPt, controllers,
                                                      oldFormat, rescan=0)
        
    parDicOffline = ngamsPlugInApi.\
                    parseRawPlugInPars(srvObj.getCfg().getOfflinePlugInPars())
    
    # This is only unmounting the NGAMS disks and may lead to problems
    # if someone mounts other disks off-line.
    if (parDicOffline.has_key("unmount")):
        unmount = int(parDicOffline["unmount"])
    else:
        unmount = 1
    if (unmount):
        try:
            ngamsLinuxSystemPlugInApi.ngamsUmount(diskDic,
                                                  srvObj.getCfg().getSlotIds())
            stat = ngamsLinuxSystemPlugInApi.rmMod(parDicOnline["module"])
            if (stat):
                errMsg = "Problem executing ngamsLinuxOfflinePlugIn! " +\
                         "The system is in not in a safe state!"
                errMsg = genLog("NGAMS_ER_OFFLINE_PLUGIN", [errMsg])
                error(errMsg)
                raise Exception, errMsg
            msg = "Kernel module " + parDicOnline["module"] + " unloaded"
            info(1,msg)
        except Exception, e:
            pass
        
        # Fallback umount.
        ngamsLinuxSystemPlugInApi.umount(rootMtPt)


if __name__ == '__main__':
    """
    Main function.
    """
    import sys
    import ngamsConfig, ngamsDb

    setLogCond(0, "", 0, "", 1)

    if (len(sys.argv) != 2):
        print "\nCorrect usage is:\n"
        print "% python ngamsLinuxOfflinePlugIn <NGAMS cfg>\n"
        sys.exit(0)    
    
    ngamsCfgObj = ngamsConfig.ngamsConfig()
    ngamsCfgObj.load(sys.argv[1])
    dbConObj = ngamsDb.ngamsDb(ngamsCfgObj.getDbServer(),
                               ngamsCfgObj.getDbName(),
                               ngamsCfgObj.getDbUser(),
                               ngamsCfgObj.getDbPassword())
    dbConObj.query("use " + ngamsCfgObj.getDbName())
    ngamsLinuxOfflinePlugIn(dbConObj, ngamsCfgObj)


# EOF
