#******************************************************************************
# ESO/DMD
#
# "@(#) $Id: ngamsLinuxOnlinePlugIn.py,v 1.32 2005/08/30 09:52:31 jknudstr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  10/05/2001  Created.
#

"""
Module containing a System Online Plug-In used by the ESO NGAS installations.
"""

from   ngams import *
import ngamsPlugInApi
import ngamsServer
import ngamsLinuxSystemPlugInApi, ngamsEscaladeUtils


def ngamsLinuxOnlinePlugIn(srvObj,
                           reqPropsObj = None):
    """
    Function mounts all NGAMS disks and loads the kernel module for the IDE
    controller card. It returns the NGAMS specific disk info dictionary.

    srvObj:        Reference to instance of the NG/AMS Server
                   class (ngamsServer).

    reqPropsObj:   NG/AMS request properties object (ngamsReqProps).

    Returns:       Disk info dictionary (dictionary).
    """
    info(4,"Entering ngamsLinuxOnlinePlugIn() ...")
    rootMtPt = srvObj.getCfg().getMountRootDirectory()
    parDic = ngamsPlugInApi.\
             parseRawPlugInPars(srvObj.getCfg().getOnlinePlugInPars())
    stat = ngamsLinuxSystemPlugInApi.insMod(parDic["module"])
    if (stat == 0):
        msg = "Kernel module " + parDic["module"] + " loaded"
        info(1, msg)

        # Old format = unfortunately some Disk IDs of WDC/Maxtor were
        # generated wrongly due to a mistake by IBM, which lead to a wrong
        # implementation of the generation of the Disk ID.
        if (not parDic.has_key("old_format")):
            raise Exception, "Missing Online Plug-In Parameter: old_format=0|1"
        else:
            oldFormat = int(parDic["old_format"])

        # The controllers Plug-In Parameter, specifies the number of controller
        # in the system.
        if (not parDic.has_key("controllers")):
            controllers = None
        else:
            controllers = parDic["controllers"]

        # Get start index for the 3ware disk devices.
        if (not parDic.has_key("dev_start_idx")):
            devStartIdx = "a"
        else:
            devStartIdx = parDic["dev_start_idx"]

        # Try first to umount possibly mounted disks (forced).
        ngamsLinuxSystemPlugInApi.umount(rootMtPt)
            
        # Select between 3ware WEB Interface and 3ware Command Line Tool.
        if (parDic["uri"].find("http") != -1):       
            diskDic = ngamsEscaladeUtils.parseHtmlInfo(parDic["uri"], rootMtPt)
        else:
            diskDic = ngamsEscaladeUtils.\
                      parseCmdLineInfo(rootMtPt,
                                       controllers,
                                       oldFormat,
                                       slotIds = ["*"],
                                       devStartIdx = devStartIdx)

        ngamsLinuxSystemPlugInApi.removeFstabEntries(diskDic)
        ngamsLinuxSystemPlugInApi.ngamsMount(srvObj, diskDic,
                                             srvObj.getCfg().getSlotIds())
        info(4,"Leaving ngamsLinuxOnlinePlugIn()")
        return diskDic
    else:
        errMsg = "Problem executing ngamsLinuxOnlinePlugIn"
        errMsg = genLog("NGAMS_ER_ONLINE_PLUGIN", [errMsg])
        error(errMsg)
        raise Exception, errMsg


if __name__ == '__main__':
    """
    Main function.
    """
    import sys
    import ngamsConfig, ngamsDb

    setLogCond(0, "", 0, "", 1)
    
    if (len(sys.argv) != 2):
        print "\nCorrect usage is:\n"
        print "% python ngamsLinuxOnlinePlugIn <NGAMS cfg>\n"
        sys.exit(0)

    srvObj = ngamsServer.ngamsServer()  
    ngamsCfgObj = ngamsConfig.ngamsConfig().load(sys.argv[1])
    dbConObj = ngamsDb.ngamsDb(ngamsCfgObj.getDbServer(),
                               ngamsCfgObj.getDbName(),
                               ngamsCfgObj.getDbUser(),
                               ngamsCfgObj.getDbPassword())
    srvObj.setCfg(ngamsCfgObj).setDb(dbConObj)
    diskDic = ngamsLinuxOnlinePlugIn(srvObj)
    slotIds = []
    for slotId in diskDic.keys(): slotIds.append(int(slotId))
    slotIds.sort()
    for slotId in slotIds:
        print "=Slot ID: %s:\n%s" % (str(slotId),
                                     diskDic[str(slotId)].dumpBuf())

# EOF
