#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: ngamsWakeUpPlugIn.py,v 1.6 2004/08/04 16:45:30 ngasmgr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  23/01/2002  Created
#

"""
Test WakeUp Plug-In to simulate the NGAS host suspension.
"""

import commands

from   ngams import *
import ngamsHighLevelLib


def ngamsWakeUpPlugIn(srvObj,
                      hostId):
    """
    Wake-Up Plug-In to wake up a suspended NGAS host.

    srvObj:         Reference to instance of the NG/AMS Server (ngamsServer).

    hostId:         Name of NGAS host to be woken up (string).
 
    Returns:        Void.
    """
    info(3,"Wake-Up Plug-In executing ...")
    hostDic = ngamsHighLevelLib.\
              getHostInfoFromHostIds(srvObj.getDb(), [hostId])
    if (not hostDic.has_key(hostId)):
        errMsg = "ngamsWakeUpPlugIn: Could not wake up host: " + hostId +\
                 " - host not defined in NGAS DB."
        raise Exception, errMsg

    networkDevs = srvObj.getCfg().getWakeUpPlugInPars()
    cmdFormat = "sudo /sbin/ether-wake -i %s -b " +\
                hostDic[hostId].getMacAddress()
    info(3,"Waking up suspended host: %s" % hostId)
    for dev in networkDevs.split(","):
        cmd = cmdFormat % dev
        info(3,"Broadcasting wake-up package - command: %s" % cmd)
        stat, out = commands.getstatusoutput(cmd)
        if (stat != 0):
            format = "ngamsWakeUpPlugIn: Problem waking up host: %s " +\
                     ". Error: %s."
            errMsg = format % (hostId, str(out).replace("\n", " "))
            raise Exception, errMsg    
    info(3,"Leaving Wake-Up Plug-In")


# EOF
