#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: ngamsCmdHandling.py,v 1.87 2004/08/04 16:45:30 ngasmgr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  07/01/2002  Created
#

"""
Contains various functions for handling commands.
"""

from   ngams import *
import ngamsArchiveCmd, ngamsCheckFileCmd, ngamsDiscardCmd
import ngamsConfigCmd, ngamsCloneCmd
import ngamsExitCmd, ngamsHelpCmd, ngamsInitCmd, ngamsLabelCmd, ngamsOfflineCmd
import ngamsOnlineCmd, ngamsRegisterCmd, ngamsRemDiskCmd, ngamsRemFileCmd
import ngamsRetrieveCmd, ngamsStatusCmd, ngamsSubscribeCmd, ngamsUnsubscribeCmd


def cmdHandler(srvObj,
               reqPropsObj,
               httpRef):
    """
    Handle a command.

    srvObj:        Reference to NG/AMS server class object (ngamsServer).
    
    reqPropsObj:   Request Property object to keep track of
                   actions done during the request handling
                   (ngamsReqProps).
        
    httpRef:       Reference to the HTTP request handler
                   object (ngamsHttpRequestHandler).
        
    Returns:       Void.
    """
    info(4,"Entering cmdHandler() ...")
    # Interpret the command + parameters.
    cmd = reqPropsObj.getCmd()
    info(1,"Received command: " + cmd)
    if (cmd == "ngamsInternal.dtd"):
        # Special handling.
        reqPropsObj.setCmd(NGAMS_RETRIEVE_CMD).addHttpPar("internal", cmd)
        ngamsRetrieveCmd.handleCmdRetrieve(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_ARCHIVE_CMD):
        ngamsArchiveCmd.handleCmdArchive(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_CHECKFILE_CMD):
        ngamsCheckFileCmd.handleCmdCheckFile(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_CLONE_CMD):
        ngamsCloneCmd.handleCmdClone(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_CONFIG_CMD):
        ngamsConfigCmd.handleCmdConfig(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_DISCARD_CMD):
        ngamsDiscardCmd.handleCmdDiscard(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_EXIT_CMD):
        ngamsExitCmd.handleCmdExit(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_HELP_CMD):
        ngamsHelpCmd.handleCmdHelp(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_INIT_CMD):
        ngamsInitCmd.handleCmdInit(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_LABEL_CMD):
        ngamsLabelCmd.handleCmdLabel(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_OFFLINE_CMD):
        ngamsOfflineCmd.handleCmdOffline(srvObj,reqPropsObj, httpRef)
    elif (cmd == NGAMS_ONLINE_CMD):
        ngamsOnlineCmd.handleCmdOnline(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_REGISTER_CMD):
        ngamsRegisterCmd.handleCmdRegister(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_REMDISK_CMD):
        ngamsRemDiskCmd.handleCmdRemDisk(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_REMFILE_CMD):
        ngamsRemFileCmd.handleCmdRemFile(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_RETRIEVE_CMD):
        ngamsRetrieveCmd.handleCmdRetrieve(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_STATUS_CMD):
        ngamsStatusCmd.handleCmdStatus(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_SUBSCRIBE_CMD):
        ngamsSubscribeCmd.handleCmdSubscribe(srvObj, reqPropsObj, httpRef)
    elif (cmd == NGAMS_UNSUBSCRIBE_CMD):
        ngamsUnsubscribeCmd.handleCmdUnsubscribe(srvObj, reqPropsObj, httpRef)
    else:
        errMsg = genLog("NGAMS_ER_ILL_CMD", [cmd])
        error(errMsg)
        raise Exception, errMsg
    info(4,"Leaving cmdHandler()")


# EOF
