#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: ngamsOfflineCmd.py,v 1.5 2004/08/04 16:45:30 ngasmgr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  13/05/2003  Created
#

"""
Function + code to handle the OFFLINE command.
"""

import time

from ngams import *
import ngamsSrvUtils 


def handleCmdOffline(srvObj,
                     reqPropsObj,
                     httpRef):
    """
    Handle an OFFLINE command.
        
    srvObj:       Reference to NG/AMS server class object (ngamsServer).
    
    reqPropsObj:  Request Property object to keep track of
                  actions done during the request handling
                  (ngamsReqProps).
        
    httpRef:      Reference to the HTTP request handler
                  object (ngamsHttpRequestHandler).
        
    Returns:      Void.
    """
    # If wait=1 is specified, send an immediate reply.
    if (not reqPropsObj.getWait()):
        srvObj.reply(reqPropsObj.setCompletionTime(), httpRef,
                     NGAMS_HTTP_SUCCESS, NGAMS_SUCCESS,
                     "Accepted command OFFLINE")
        time.sleep(0.250)

    if (not reqPropsObj.hasHttpPar("force")):
        srvObj.checkSetState("Command OFFLINE", [NGAMS_ONLINE_STATE],
                             [NGAMS_IDLE_SUBSTATE], NGAMS_OFFLINE_STATE)
    else:
        srvObj.checkSetState("Command OFFLINE", [NGAMS_ONLINE_STATE],
                             [NGAMS_IDLE_SUBSTATE, NGAMS_BUSY_SUBSTATE],
                             NGAMS_OFFLINE_STATE)
    ngamsSrvUtils.handleOffline(srvObj, reqPropsObj)
    srvObj.reply(reqPropsObj.setCompletionTime(), httpRef, NGAMS_HTTP_SUCCESS,
                 NGAMS_SUCCESS, "Successfully handled command OFFLINE")
    srvObj.updateRequestDb(reqPropsObj)
    info(1,"Successfully handled command OFFLINE!")


# EOF
