#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: ngamsUnsubscribeCmd.py,v 1.3 2004/08/04 16:45:30 ngasmgr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  05/11/2002  Created
#

"""
This module contains functions used in connection with the
UNSUBSCRIBE Command.
"""

import pcc, PccUtTime
from ngams import *
import ngamsLib, ngamsSubscriber


def delSubscriber(srvObj,
                  subscrId):
    """
    Remove a Susbcriber from the internal list + from the DB.

    srvObj:      Reference to NG/AMS Server object (ngamsServer).

    subscrId:    Subscriber ID (string).

    Returns:     Void.
    """
    info(4,"Entering delSubscriber() ...")
    err = 0
    try:
        srvObj.getDb().deleteSubscriber(subscrId)
    except Exception, e:
        warning("Error deleting Subscriber information from the DB. " +\
                "Subscriber ID: " + subscrId + ". Exception: " + str(e))
        err = 1
    try:
        del srvObj.getSubscriberDic()[subscrId]
    except Exception, e:
        warning("Error deleting Subscriber information kept internally. " +\
                "Subscriber ID: " + subscrId + ". Exception: " + str(e))
        err = 1
    if (not err):
        info(2,"Subscriber with ID: " + subscrId +\
             " successfully unsubscribed")
    info(4,"Leaving delSubscriber() ...")


def handleCmdUnsubscribe(srvObj,
                         reqPropsObj,
                         httpRef):
    """
    Handle UNSUBSCRIBE command.

    srvObj:         Reference to NG/AMS server class object (ngamsServer).

    reqPropsObj:    Request Property object to keep track of actions done
                    during the request handling (ngamsReqProps).
        
    httpRef:        Reference to the HTTP request handler
                    object (ngamsHttpRequestHandler).

    Returns:        Void.
    """
    info(4,"Entering handleCmdUnsubscribe() ...")
    if (reqPropsObj.hasHttpPar("url")):
        url = reqPropsObj.getHttpPar("url")
    else:
        errMsg = genLog("NGAMS_ER_CMD_SYNTAX",
                        [NGAMS_SUBSCRIBE_CMD, "Missing parameter: url"])
        raise Exception, errMsg
    delSubscriber(srvObj, ngamsLib.getSubscriberId(url))

    srvObj.reply(reqPropsObj, httpRef, NGAMS_HTTP_SUCCESS, NGAMS_SUCCESS,
                 "Handled UNSUBSCRIBE command")
    info(4,"Leaving handleCmdUnsubscribe()")


# EOF
