/******************************************************************************
 * ESO/DMD
 *
 * "@(#) $Id: ngamsCreateOpsLogTable.sql,v 1.7 2004/08/11 11:42:12 ngasmgr Exp $"
 *
 * Who       When        What
 * --------  ----------  -----------------------------------------------------
 * jknudstr  17/12/2003  Created
 */

/* isql -SELECT<DB Server> -Ungas -P***** < ngamsCreateOpsLogTable.sql
 */

print "Using database ngas"
use ngas
go

print "Dropping old ngas_ops_log_book table"
drop table ngas_ops_log_book
go

/* --------------------------------------------------------------------- */
print "Creating ngas_ops_log_book table"
go
/*-
 * Table:
 *   NGAS Operators Log Book Table - ngas_ops_log_book
 *
 * Description:
 *   The table is used by the NGAS Operators to log all maintenance actions
 *   that have been performed on the system. In this way it is possible to
 *   trace if a disk e.g. was sent back to the vendor for repair etc.
 *
 * Columns:
 *   action_date:   Date the action was performed.
 *
 *   ref_id:        Reference to the event. If the item in question is a file,
 *                  this should be the File ID. If the item is a disk, this
 *                  should be the Disk ID.
 *
 *   action:        Action carried out, e.g. "Removed Disk".
 *
 *   operator_id:   ID of the operator initiating the action.
 *
 *   comment:       Optional comment.
 *
 *   remarks:       Other remarks or information in connection with the event.
-*/
Create table ngas_ops_log_book
(
	action_date             datetime	not null,
	ref_id			varchar(128)	not null,
	action			varchar(64)	not null,
	operator_id		varchar(64)	not null,
	comment	 		varchar(128)	null,
	remarks			text		null
)
go
print "Create index on ngas_ops_log_book"
create unique clustered index dcode_cluster on ngas_ops_log_book(action_date, ref_id)
go
print "Granting"
grant insert, update, delete, select on ngas_ops_log_book to ngas
go
grant select on ngas_ops_log_book to public
go

/* EOF */
