#******************************************************************************
# ESO/DFS
#
# "@(#) $Id: ngamsHelpCmdTest.py,v 1.12 2005/01/25 10:29:38 ngasmgr Exp $"
#
# Who       When        What
# --------  ----------  -------------------------------------------------------
# jknudstr  18/11/2003  Created
#

"""
This module contains the Test Suite for the HELP Command.
"""


import os, sys
from   ngams import *
from   ngamsTestLib import *


class ngamsHelpCmdTest(ngamsTestSuite):
    """
    Synopsis:
    Test Suite of the HELP Command.

    Description:
    The purpose of the Test Suite is to exercise the HELP Command.

    Missing Test Cases:
    NOTE: The HELP Command is not yet implemented. When implemented this
          Test Suite should be reviewed and the missing Test Cases added.
    """

    def test_NoPars_1(self):
        """
        Synopsis:
        Issue HELP Command with no parameters/Offline State.
        
        Description:
        Check that the response to the HELP Command is as expected (taking
        into account that the HELP Command is not yet implemented).

        Expected Result:
        The server should send back an Error Response indicating that the
        HELP Command is not yet implemented.
        
        Test Steps:
        - Start server.
        - Issue HELP Command.
        - Check that output is as expected (=command rejected).
            
        Remarks:
        This Test Case should be modified when the HELP Command has been
        implemented.
        """
        self.prepExtSrv(8888, 1, 1, 0)
        tmpStatFile = sendExtCmd(getHostName(), 8888, NGAMS_HELP_CMD)
        refStatFile = "ref/ngamsHelpCmdTest_test_NoPars_1_1_ref"
        self.checkFilesEq(refStatFile, tmpStatFile,
                          "Incorrect status returned for HELP command")


    def test_NoPars_2(self):
        """
        Synopsis:
        Issue HELP Command with no parameters/Online State.
        
        Description:
        Check that the response to the HELP Command is as expected (taking
        into account that the HELP Command is not yet implemented).
        
        Expected Result:
        The server should send back an Error Response indicating that the
        HELP Command is not yet implemented.

        Test Steps:
        - Start server.
        - Issue HELP Command.
        - Check that output is as expected (command rejected).
            
        Remarks:
        This Test Case should be modified when the HELP Command has been
        implemented.
        """   
        self.prepExtSrv(8888, 1, 1, 1)
        tmpStatFile = sendExtCmd(getHostName(), 8888, NGAMS_HELP_CMD)
        refStatFile = "ref/ngamsHelpCmdTest_test_NoPars_2_1_ref"
        self.checkFilesEq(refStatFile, tmpStatFile,
                          "Incorrect status returned for HELP command")


def run():
    """
    Run the complete test.

    Returns:   Void.
    """
    runTest(["ngamsHelpCmdTest"])


if __name__ == '__main__':
    """
    Main program executing the test cases of the module test.
    """
    runTest(sys.argv)


# EOF
